package web.controller;

import core.service.BaseService;
import db.AbstractModel;
import db.DBAction;
import db.JsonResult;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.dao.DataAccessException;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;

/**
 * Created by reborn on 2017/9/14.
 */
public abstract class BaseController<T extends AbstractModel>{
    protected static Logger log=LogManager.getLogger();

    @Resource
    public BaseService baseService;

    @ResponseBody
    @RequestMapping(value = "{action}", produces = MediaType.APPLICATION_JSON_VALUE)
    public <T extends AbstractModel> JsonResult curd(@PathVariable DBAction action, T model) {
        log.info(model.tableNote()+ "进行" + action.getCh() + "操作请求");
        if(checkAction(action)) {
            try {
                return new JsonResult<T>(baseService.curd(action, model),JsonResult.type.success,action);
            }catch (DataAccessException e){
                log.error(e);
                return new JsonResult<String>("非法操作",JsonResult.type.fail,action);
            }
        }else{
            return new JsonResult<>(model.tableNote() + "不允许" + action.getCh() + "操作", JsonResult.type.fail,action);
        }

    }

    @ResponseBody
    @RequestMapping(value = "batch/{action}", produces = MediaType.APPLICATION_JSON_VALUE)
    public <T extends AbstractModel> JsonResult find(@PathVariable DBAction action, T queryCommand,@RequestParam Integer firstResult,@RequestParam Integer maxResults){
        log.info(queryCommand.tableNote()+ "进行批量" + action.getCh() + "操作请求");
        if(checkAction(action)) {
            try {
                return new JsonResult<>(baseService.find(queryCommand, firstResult, maxResults), JsonResult.type.success, action);
            } catch (DataAccessException e) {
                log.error(e);
                return new JsonResult<String>("非法操作",JsonResult.type.fail,action);
            }
        }else{
            return new JsonResult<>(queryCommand.tableNote() + "不允许" + action.getCh() + "操作", JsonResult.type.fail,action);
        }

    }


    public abstract boolean checkAction(DBAction action);
}
